﻿using System;
using System.Runtime.CompilerServices;
using System.Security.Permissions;
using System.Threading;
using System.Windows.Threading;
using Dex2;
using EikonDesktopDataAPI;

namespace ComApiUsageExampleDexMultiThreading.Services
{
    public class ReferenceDataService:IDisposable
    {

        private ManualResetEvent resetEventInitialize = new ManualResetEvent(false);
        private ManualResetEvent resetEventRequest = new ManualResetEvent(false);

        public ReferenceDataService()
        {
            InitializeEikonApi();
            Dispatcher.PushFrame(Program.GetDispatcherFrame());
            resetEventInitialize.WaitOne();
        }

        public string FieldValue(string instrumentCode, string fieldName)
        {
            _request = _dex2Mgr.CreateRData(_cookie);
            _request.OnUpdate += _request_OnUpdate;

            _request.InstrumentIDList = new[] { instrumentCode };
            _request.FieldList = new[] {fieldName};
            _request.Subscribe(false);

            Console.WriteLine("Requesting {0} for {1} ...", fieldName, instrumentCode);

            Dispatcher.PushFrame(Program.GetDispatcherFrame());
            resetEventRequest.WaitOne();
            
            return _request.Data[0, 0];
        }

        private void _request_OnUpdate(DEX2_DataStatus dataStatus, object error)
        {   
            resetEventRequest.Set();
            Program.StopMessagePump();
        }

        #region Desktop Data API

        private readonly EikonDesktopDataAPI.EikonDesktopDataAPI EikonApi = new EikonDesktopDataAPI.EikonDesktopDataAPI();

        private static Dex2.Dex2Mgr _dex2Mgr;
        private static Dex2.RData _request;
        private static int _cookie;


        private void InitializeEikonApi()
        {
            switch (EikonApi.Initialize())
            {
                case EEikonDataAPIInitializeResult.Succeed:
                    Console.WriteLine("Eikon API initialized. Connecting...");
                    break;
                case EEikonDataAPIInitializeResult.Error_Reinitialize:
                    Console.WriteLine("Eikon API reinit failed.");
                    break;
                case EEikonDataAPIInitializeResult.Error_InitializeFail:
                    Console.WriteLine("Eikon API initialization failed.");
                    break;
            }

            EikonApi.OnStatusChanged += EikonApiOnOnStatusChanged;

        }

        private void EikonApiOnOnStatusChanged(EEikonStatus eStatus)
        {
            Console.WriteLine("Eikon API status updated. New status: {0}", eStatus.ToString());

            switch (eStatus)
            {
                case EEikonStatus.Disconnected:
                    break;
                case EEikonStatus.Connected:
                      _dex2Mgr = EikonApi.CreateDex2Mgr();
                      _cookie = ((Dex2.IDex2Mgr2)_dex2Mgr).Initialize(DEX2_MetadataCaller.DE_MC_ADC_UNKNOWN);
                      Console.WriteLine("Dex initialized.");
                    break;
                case EEikonStatus.LocalMode:
                    break;
                case EEikonStatus.Offline:
                    break;
                case EEikonStatus.Disconnected_No_license_agreement_sign_off:
                    break;
                default:
                    throw new ArgumentOutOfRangeException("eStatus");
            }

            resetEventInitialize.Set();
            Program.StopMessagePump();
        }
        #endregion

        public void Dispose()
        {
            if (_dex2Mgr != null)
            {
                _dex2Mgr.Finalize(_cookie);
                _dex2Mgr = null;
            }
        }
    }
}
